#ifndef datei_h
#define datei_h

#include <iostream.h>
#include <fstream.h>
#include <IV-look/dialogs.h>

#include "lingvar.h"
#include "regel.h"
#include "nfclook.h"

/*
 *------------------------------------------------------------------------------
 * Klasse : DateiArbeit
 * Zweck  : Dient dem Abspeicher und Laden von linguistischen Variablen und
 *          Regeln in/aus Dateien.
 *------------------------------------------------------------------------------
 */
class DateiArbeit
{
  public : DateiArbeit(Window*, NfcAusgabe*);
           void speichern(boolean steuer_var, const LingVarSatz*);
           void speichern(boolean steuer_regeln, const RegelBasis*);
           void laden(boolean steuer_var, LingVarSatz*);
           void laden(boolean steuer_regeln, RegelBasis*);

  // lokale Funktionen und Variablen
  private: enum Modus { LvSpeichern, LvLaden, RgSpeichern, RgLaden };
           void _ausfuehren(Modus);
           void _callback(FileChooser*, boolean ok);
           CopyString _parse(String eingabe, String extension);
           Modus _modus;
           String _lv_extension,           // Datei-Endungen
                  _rg_extension,
                  _lv_filter,
                  _rg_filter;

           const LingVarSatz* _quell_vars;
           LingVarSatz* _ziel_vars;

           const RegelBasis* _quell_regeln;
           RegelBasis* _ziel_regeln;

           // Handelt es sich um Steuer- oder Fehler-Var/Regeln ?
           // ist nur wichtig fuer Meldung im Ausgabefenster.
           boolean _steuer_daten;

           Window* _basis_fenster;
           NfcAusgabe* _ausgabe_feld;
};

  // Deklartionen fuer die Ein/Ausgabe-Operatoren fuer ling. Variablen
  ofstream& operator<<(ofstream& os, FsIntervall*);
  istream& operator>>(istream& is, FsIntervall*);

  ofstream& operator<<(ofstream& os, FuzzySet*);
  istream& operator>>(istream& is, FuzzySet*);

  ofstream& operator<<(ofstream& os, LingVar*);
  istream& operator>>(istream& is, LingVar*);

  ofstream& operator<<(ofstream& os, const LingVarSatz*);
  istream& operator>>(istream& is, LingVarSatz*);

  // Deklartionen fuer die Ein/Ausgabe-Operatoren fuer Regeln
  ofstream& operator<<(ofstream& os, const RegelBasis*);
  istream& operator>>(istream& is, RegelBasis*);

  ofstream& operator<<(ofstream& os, Regel*);
  istream& operator>>(istream& is, Regel*);

  // Hilfsfunktion zum Loeschen von Trennzeichen
  void del_trenn(istream& is);

#endif
